/*
  ICFS: Interactively Controlled File System
  Copyright (C) 2024-2025  Fedir Kovalov

  This program can be distributed under the terms of the GNU GPLv2.
  See the file LICENSE.
*/

#ifndef UI_SOCKET_H
#define UI_SOCKET_H

#include "process_info.h"
#include <sys/types.h>

/**
 * Initialize UI socket and required permission tables
 *
 * @param perm_permissions_db_filename Path to persistent permissions DB
 * @return 0 on success, 1 on failure
 */
int init_ui_socket(const char *perm_permissions_db_filename);

/**
 * Clean up UI socket resources
 */
void destroy_ui_socket(void);

/**
 * Determine file access based on permission tables and user input
 *
 * Checks permissions in order:
 * 1. Temporary permission table
 * 2. Permanent permission table
 * 3. User decision via GUI
 *
 * @param filename File being accessed
 * @param pi Process information
 * @param opts Flags to force permission (GRANT_TEMP/GRANT_PERM)
 * @return 1 if access allowed, 0 if denied
 */
int interactive_access(const char *filename, struct process_info pi, int opts);

#define GRANT_TEMP 1
#define GRANT_PERM 2
// #define TABLE_ONLY 4 // NOTE: Add this in the future?

#endif // !UI_SOCKET_H
